% implements method M4 from the paper
% "Filtered Leapfrog Time Integration with Enhanced Stability Properties".
% for autonomous systems of ODEs

% This is a reference implementation only, which was written for clarity.
% The implementation is not in any way optimized for performance or 
% storage requirements.

 function [u,t] = M4b_MOL(k,F,T,y0,R,M,C)
 
 % u3 = u(n-3)   needed for filtering u(n-1)
 % u2 = u(n-2)
 % u1 = u(n-1)
 %  u = u(n)
 % p1 = u(n+1)
 % p2 = u(n+2)
 
 %  ---- start with the IC ---------------------------------------------
 
    u2 = y0;
    t = k;
    
 % -- first step takes M substeps -----------------

    u1 = y0 + k*feval(F,y0)/M;   % one Euler
    e1 = u2;
    
    for j=2:M
         et = u1;
         u1 = e1 + 2*(k/M)*feval(F,u1);  % M-1 leapfrog steps
         e1 = et;
    end


     for j = 2:R
        t = t + k; 
        u = u2 + 2*k*feval(F,u1);
        if j ~= R          % get ready for the next step
            u3 = u2;
            u2 = u1;
            u1 = u;
        end
     end
    
    % prepare to filter, calculate 2 auxillary time levels
    p1 = u1 + 2*k*feval(F,u);
    p2 = u + 2*k*feval(F,p1);
     
    for c = 1:C
         % apply filter to time level n-1 and n
         u1 = ( -u3 + 4*u2 + 10*u1 + 4*u  - p1 )/16;
          u = ( -u2 + 4*u1 + 10*u  + 4*p1 - p2 )/16;
          
          u3 = u2;  % get ready for next time step
          u2 = u1;
          u1 = u;
         
        for j = 1:R
              if t>=(T - 10000*eps)
                 return
              end
              t = t + k; 
              u = u2 + 2*k*feval(F,u1);
            if j~=R
                u3 = u2;
                u2 = u1;
                u1 = u;
            end
         end  

    p1 = u1 + 2*k*feval(F,u);
    p2 = u + 2*k*feval(F,p1);
    u = ( -u2 + 4*u1 + 10*u  + 4*p1 - p2 )/16; % filter
              
    end % C
    
 %  ---- restart using u as the IC -----------------------------------------    
    
   % the above C loop has exited without updating for the next step, so 
   %  do so now
   
    u3 = u2;
    u2 = u1;
    u1 = u;

    while (t<T)
         if t>=(T - 10000*eps)
             return
         end
         t = t + k;
     
          ut3 = u3;
          ut2 = u2;
          ut1 = u1;

          ut = ut1 + k*feval(F,ut1)/M;
          
          ut3 = ut2;
          ut2 = ut1;
          ut1 = ut;
         
          for j=2:M
            ut = ut2 + 2*(k/M)*feval(F,ut1); 
            ut3 = ut2;
            ut2 = ut1;
            ut1 = ut;
          end
          
          u = ut;
          u3 = u2;
          u2 = u1;
          u1 = u;
          
         for j = 2:R
              if t>=(T - 10000*eps)
                 return
              end
              t = t + k;
         
            u = u2 + 2*k*feval(F,u1);
             if j~=R
                u3 = u2;
                u2 = u1;
                u1 = u;
            end
         end 
         
           % prepare to filter, calculate 2 auxillary time levels
            p1 = u1 + 2*k*feval(F,u);
            p2 = u + 2*k*feval(F,p1);

    for c = 1:C
         % apply filter to time level n-1 and n
         u1 = ( -u3 + 4*u2 + 10*u1 + 4*u  - p1 )/16;
          u = ( -u2 + 4*u1 + 10*u  + 4*p1 - p2 )/16;
          
          u3 = u2;
          u2 = u1;
          u1 = u;

       for j = 1:R
              if t>=(T - 10000*eps)
                 return
              end
              t = t + k;
              u = u2 + 2*k*feval(F,u1);
              
             if j~=R
                u3 = u2;
                u2 = u1;
                u1 = u;
            end
         end  

          p1 = u1 + 2*k*feval(F,u);
          p2 = u + 2*k*feval(F,p1);
           u = ( -u2 + 4*u1 + 10*u  + 4*p1 - p2 )/16;  % filter
    end % C
    
    % the above C loop has exited without updating for the next step, so 
    %  do so now
   
    u3 = u2;
    u2 = u1;
    u1 = u;
        
  end % while
    
    
    
    
    
    
    
    
    
    
