function advection_Fourier_M4()

% reproduces the advection equation example, example 3, from the paper
% "Filtered Leapfrog Time Integration with Enhanced Stability Properties".
% The script will reproduce the values in tables 4 and 5.


% Fourier pseudospectral, periodic boundary conditions
%  Ut + Ux = 0   on [0,1]
%    U(x,0) = (sin(pi*x).^40)
        
    format compact, format short e
    A = 0;  B = 1;      % interval [A,B]
    N = 179;
    
    R = 20;    % restart every R steps
    M = 4;     % restarting substeps
    C = 2;     % continue C times without starting
    
% function fourdif.m is from the Matlab DMSUITE which is freely availbe at:
% http://www.mathworks.com/matlabcentral/fileexchange/29
    
    [x,D] = fourdif(N,1);
    D = diag(2.*pi.*ones(N,1))*D;
    kk = 1:N; x = 0 + (kk-1)/N;    x = x'; 
    
    ew = eig(D);
    mew = max(imag(ew));
%    dt = 0.45/mew
    dt = 0.7/mew
    T = 1;
    
     u0 = exactSolution(x,0);  
     [u,FT] = M4_MOL(dt,@F,T,u0,R,M,C);
     exact = exactSolution(x,FT);

     plot(x,u,'b',x,exact,'g--')
     axis([0 1 -0.2 1.05])
     xlabel('x')
     ylabel('u(x,t)')

     disp('max error of: '), eInf = norm(u-exact,inf)
     disp('at time t = '), FT


   function fp = F(v)
      fp = D*v;  
   end
   
    function ex = exactSolution(x,t)
      ex = sin((x+t)*pi).^(40);
   end
         


end

   




  
  
