
% numerically calculates the rate of convergence of method M4 from the paper
% "Filtered Leapfrog Time Integration with Enhanced Stability Properties"
% on a scalar test problem

function convergence_M4()
% 
% y' = -y
%   y(0) = 1
% exact y(t) = exp(-t)
        
 format compact % , format long

 T = 1;
 Nv = [20 40 80 160 320 640 1280 2560 5120];
 y0 = 1;
      
  R = 20;    % restart every R steps
  M = 4;     % restarting substeps
  C = 2;     % continue C times without starting
   
  for j=1:length(Nv)
       N = Nv(j);
       k = T/(N-1); 
       v = M4(k,@F,T,y0,7,M,C);
       y = v(1:N);
       er(j) = abs( exp(-T) - y(N) );
       DT(j) = k;
   end % j
       
   loglog(DT,er,'b'); 
   xlabel '\Delta t', ylabel '|error|', title('method M4 convergence plot')

    rho = zeros(length(Nv)-1,1);
    for i=2:length(Nv);
       rho(i-1) = log(er(i-1)./er(i))/log(2);          % rate of convergence
    end
    display('M4 rate of convergence: '), rho

   function f = F(v)
      f = -v;
   end

end



  
  
