
% plots the absolute stability region of method M4 from the paper
% "Filtered Leapfrog Time Integration with Enhanced Stability Properties".
%
% The code is compatible with the Matlab version 7 symbolic toolbox.  In
% later versions of Matlab a different symbolic toolbox is used.
%
% The scipt reproduces the 2 stability plots in figure 6 of the manuscript

function  stability_M4()

   M = 4; C = 2;  N = 7; 
   xmax = 0.02; xmin = -0.6; ymax = 1.1;   
%   xmax = 0.01; xmin = -0.01; ymax = 1.1;   % zoom in on imaginary axis
   syms z y
   
   y(1) = 1;
   y(2) = simplify( 1 + z/M );       % 1/M Euler
   e1 = y(1);
   for j=2:M
      et = y(2);
      y(2) = simplify( e1 + 2*z*y(2)/M );   % (M-1) 1/M leapform
      e1 = et;
   end
                                               
   for n=3:N+2
      y(n) = simplify( y(n-2) + 2*z*y(n-1) );                           
   end 
             
   for c=1:C  
       y(c*N-1) = simplify( ( -y(c*N-3) + 4*y(c*N-2) + 10*y(c*N-1) + 4*y(c*N) - y(c*N+1) )/16 );  % filter
       y(c*N) = simplify( ( -y(c*N-2) + 4*y(c*N-1) + 10*y(c*N) + 4*y(c*N+1) - y(c*N+2) )/16 );
             
       for n=c*N+1:(c+1)*N+2
           y(n) = simplify( y(n-2) + 2*z*y(n-1) );                           
       end                        
       y((c+1)*N) = simplify( ( -y((c+1)*N-2) + 4*y((c+1)*N-1) + 10*y((c+1)*N) + 4*y((c+1)*N+1) - y((c+1)*N+2) )/16 ); % filter    
   end
            
    yn = y((C+1)*N);
    sPoly5 = sym2poly(yn);
            
    x = linspace(xmin,xmax,400);
    y = linspace(-ymax,ymax,400);
    [X,Y] = meshgrid(x,y);
    z = X + i*Y;
  
    pz = polyval(sPoly5,z);
    contour(X,Y,abs( pz ),[1 1],'b');
    hold on
    plot(zeros(1,100),linspace(-1,1),'k-')
    xlabel('real(\lambda \Delta t)'); ylabel('imag(\lambda \Delta t)');
    hold off
